@@ -32,8 +32,7 @@ class Calendar(GooCanvas.Canvas):
             "The color of the body", GObject.ParamFlags.READWRITE),
         'today-body-color': (GObject.TYPE_STRING, 'ivory', "Today Body Color",
             "The color of the today body", GObject.ParamFlags.READWRITE),
-        # TODO font
-        'font-desc': (GObject.TYPE_PYOBJECT, "Font Description",
+        'font': (GObject.TYPE_STRING, '', "Font",
             "The attributes specifying which font to use",
             GObject.ParamFlags.READWRITE),
         }
@@ -41,6 +40,7 @@ class Calendar(GooCanvas.Canvas):
     def __init__(self, event_store=None, view="month", time_format="%H:%M",
             firstweekday=calendar.SUNDAY):
         super(Calendar, self).__init__()
+        settings = Gtk.Settings.get_default()
         self.__props = {
             'text-color': '#2E3634',
             'inactive-text-color': '#8B8F8E',
@@ -49,8 +49,8 @@ class Calendar(GooCanvas.Canvas):
             'inactive-border-color': '#E8E7E8',
             'body-color': 'white',
             'today-body-color': 'ivory',
+            'font': settings.get_property('gtk-font-name'),
             }
-        self._font_desc = None
         self._selected_day = None
         self._bg_rect = None
         self._timeline = None
@@ -109,19 +109,15 @@ class Calendar(GooCanvas.Canvas):
             self.days.append(box)
 
     def do_set_property(self, prop, value):
-        if prop.name == 'font-desc':
-            self._font_desc = value
-        else:
-            self.__props[prop.name] = value
+        self.__props[prop.name] = value
 
     def do_get_property(self, prop):
-        if prop.name == 'font-desc':
-            if self._font_desc is None:
-                self._font_desc = self.get_style_context().get_property(
-                    'font', Gtk.StateFlags.NORMAL).copy()
-            return self._font_desc
-        else:
-            return self.__props[prop.name]
+        return self.__props[prop.name]
+
+    @property
+    def font_size(self):
+        return self.get_style_context().get_property(
+            'font', Gtk.StateFlags.NORMAL).get_size()
 
     def select(self, new_date):
         cal = calendar.Calendar(self.firstweekday)
@@ -136,7 +132,9 @@ class Calendar(GooCanvas.Canvas):
         elif self.view == "week":
             old_first_weekday = util.first_day_of_week(cal, old_date)
             new_first_weekday = util.first_day_of_week(cal, new_date)
-            page_changed = old_first_weekday != new_first_weekday
+            page_changed = (
+                old_first_weekday != new_first_weekday
+                or old_date.month != new_date.month)
         elif self.view == "day":
             page_changed = old_date != new_date
 
@@ -205,7 +203,7 @@ class Calendar(GooCanvas.Canvas):
         if self._event_store:
             self._event_store.disconnect(self._event_removed_sigid)
             self._event_store.disconnect(self._event_added_sigid)
-            self._event_store_disconnect(self._events_cleared_sigid)
+            self._event_store.disconnect(self._events_cleared_sigid)
 
         # Set and connect new event_store
         self._event_store = event_store
@@ -253,14 +251,13 @@ class Calendar(GooCanvas.Canvas):
         """
         Draws the currently selected day.
         """
-        pango_size = self.props.font_desc.get_size()
         x, y, w, h = self.get_bounds()
         timeline_w = self._timeline.width
         dayno = self.selected_date.weekday()
         day_name = calendar.day_name[dayno]
         # Sum the needed space for the date before the day_name
         caption_size = len(day_name) + 3
-        day_width_min = caption_size * pango_size / Pango.SCALE
+        day_width_min = caption_size * self.font_size / Pango.SCALE
         day_width_max = (w - timeline_w)
         self._day_width = max(day_width_min, day_width_max)
         self._day_height = h
@@ -274,11 +271,11 @@ class Calendar(GooCanvas.Canvas):
         weeks = util.my_monthdatescalendar(cal, self.selected_date)
         for weekno, week in enumerate(weeks):
             # Hide all days that are not part of the current day
+            for i, date in enumerate(week):
+                box = self.days[weekno * 7 + i]
+                box.set_property(
+                    'visibility', GooCanvas.CanvasItemVisibility.INVISIBLE)
             if self.selected_date not in week:
-                for dayno, date in enumerate(week):
-                    box = self.days[weekno * 7 + dayno]
-                    box.set_property(
-                        'visibility', GooCanvas.CanvasItemVisibility.INVISIBLE)
                 continue
 
             if self.selected_date == datetime.date.today():
@@ -307,12 +304,11 @@ class Calendar(GooCanvas.Canvas):
         """
         Draws the currently selected week.
         """
-        pango_size = self.props.font_desc.get_size()
         x, y, w, h = self.get_bounds()
         timeline_w = self._timeline.width
         caption_size = max(len(day_name) for day_name in calendar.day_name)
         caption_size += 3  # The needed space for the date before the day_name
-        day_width_min = caption_size * pango_size / Pango.SCALE
+        day_width_min = caption_size * self.font_size / Pango.SCALE
         day_width_max = (w - timeline_w) / 7
         self._day_width = max(day_width_min, day_width_max)
         self._day_height = h
@@ -372,10 +368,9 @@ class Calendar(GooCanvas.Canvas):
         Draws the currently selected month.
         """
         x1, y1, w, h = self.get_bounds()
-        pango_size = self.props.font_desc.get_size()
         caption_size = max(len(day_name) for day_name in calendar.day_name)
         caption_size += 3  # The needed space for the date before the day_name
-        day_width_min = caption_size * pango_size / Pango.SCALE
+        day_width_min = caption_size * self.font_size / Pango.SCALE
         day_width_max = w / 7
         self._day_width = max(day_width_min, day_width_max)
         self._day_height = h / 6
@@ -441,7 +436,7 @@ class Calendar(GooCanvas.Canvas):
 
     def _get_day_item(self, find_date):
         cal = calendar.Calendar(self.firstweekday)
-        weeks = util.my_monthdatescalendar(cal, find_date)
+        weeks = util.my_monthdatescalendar(cal, self.selected_date)
         for weekno, week in enumerate(weeks):
             for dayno, date in enumerate(week):
                 if date == find_date:
@@ -474,8 +469,8 @@ class Calendar(GooCanvas.Canvas):
             return days
         raise Exception('Days not found: %s %s' % (event.start, end))
 
-    def _find_free_line(self, days):
-        for line in range(days[0].n_lines):
+    def _find_free_line(self, days, n_lines):
+        for line in range(n_lines):
             free = True
             for day in days:
                 if line in day.lines:
@@ -487,7 +482,6 @@ class Calendar(GooCanvas.Canvas):
 
     def draw_events(self):
         _, _, bound_width, _ = self.get_bounds()
-        timeline_witdh = self._timeline.width
         # Clear previous events.
         for item in self._event_items:
             item.remove()
@@ -522,7 +516,7 @@ class Calendar(GooCanvas.Canvas):
         events.sort(key=util.event_days, reverse=True)
 
         # Draw all-day events, longest event first.
-        max_y = self._selected_day.line_height
+        max_y = 0
         non_all_day_events = []
         for event in events:
             event.event_items = []
@@ -534,7 +528,10 @@ class Calendar(GooCanvas.Canvas):
 
             # Find a line that is free in all of the days.
             days = self._get_day_items(event)
-            free_line = self._find_free_line(days)
+            n_lines = days[0].n_lines
+            if self.view in {"week", "day"}:
+                n_lines = min(n_lines // 2, max(n_lines - 24, 0))
+            free_line = self._find_free_line(days, n_lines)
             if free_line is None:
                 for day in days:
                     day.show_indic = True
@@ -547,7 +544,9 @@ class Calendar(GooCanvas.Canvas):
             all_day_events_height += 1  # 1px padding-top
             max_y = max(all_day_events_height, max_y)
             for day in days:
-                day.lines[free_line] = 1
+                for i in range(free_line,
+                        free_line + len(event.caption.splitlines())):
+                    day.lines[i] = 1
 
             # Split days into weeks.
             weeks = []
@@ -578,8 +577,8 @@ class Calendar(GooCanvas.Canvas):
                 self._event_items.append(event_item)
                 self.get_root_item().add_child(event_item, -1)
                 if self.view == "day":
-                    x_start = timeline_witdh
-                    width = bound_width
+                    x_start = self._timeline.width
+                    width = bound_width - self._timeline.width
                 else:
                     x_start = day.x
                     width = (day.width + 2) * len(week)
@@ -609,6 +608,9 @@ class Calendar(GooCanvas.Canvas):
                     event_item.width -= 6
                     event_item.type = 'leftright'
                 event_item.update()
+        # Add the day title
+        if self._selected_day:
+            max_y += self._selected_day.line_height
 
         if self.view == "month":
             return
@@ -617,9 +619,10 @@ class Calendar(GooCanvas.Canvas):
         self._timeline.set_property(
             'visibility', GooCanvas.CanvasItemVisibility.VISIBLE)
         x, y, w, h = self.get_bounds()
+        max_y += (h - max_y) % 24
         self._timeline.x = x
         self._timeline.y = max_y
-        self._timeline.height = h - max_y - 2
+        self._timeline.height = h - max_y
         self._timeline.line_color = self.props.body_color
         self._timeline.bg_color = self.props.border_color
         self._timeline.text_color = self.props.text_color
@@ -689,8 +692,8 @@ class Calendar(GooCanvas.Canvas):
                     y_off1 = top_offset_mins * self.minute_height
                     y_off2 = bottom_offset_mins * self.minute_height
                     if self.view == "day":
-                        x_start = timeline_witdh
-                        column_width = w / parallel
+                        x_start = self._timeline.width
+                        column_width = (w - self._timeline.width) / parallel
                     else:
                         column_width = day.width / parallel
                         x_start = day.x
@@ -743,10 +746,11 @@ class Calendar(GooCanvas.Canvas):
         gtk_settings = Gtk.Settings.get_default()
         double_click_distance = gtk_settings.props.gtk_double_click_distance
         double_click_time = gtk_settings.props.gtk_double_click_time
-        if (self._last_click_x is not None and
-                event.time < (self._last_click_time + double_click_time) and
-                abs(event.x - self._last_click_x) <= double_click_distance and
-                abs(event.y - self._last_click_y) <= double_click_distance):
+        if (self._last_click_x is not None
+                and event.time < (self._last_click_time + double_click_time)
+                and abs(event.x - self._last_click_x) <= double_click_distance
+                and abs(event.y - self._last_click_y) <= double_click_distance
+                ):
             self._last_click_x = None
             self._last_click_y = None
             self._last_click_time = None
@@ -793,81 +797,86 @@ class Calendar(GooCanvas.Canvas):
     @util.left_click
     def on_event_item_button_press_event(self, event_item, rect, event):
 
-        # Drag and drop starting coordinates
-        self._drag_x = event.x
-        self._drag_y = event.y
-        self._drag_height = 0
-        self._drag_start_date = self.get_cur_pointed_date(event.x, event.y)
-        self._drag_date = self._drag_start_date
-        self.set_has_tooltip(False)
-        event_item.raise_(None)
-        event_item.transparent = True
+        if event_item.event.editable:
+            # Drag and drop starting coordinates
+            self._drag_x = event.x
+            self._drag_y = event.y
+            self._drag_height = 0
+            self._drag_start_date = self.get_cur_pointed_date(event.x, event.y)
+            self._drag_date = self._drag_start_date
+            self.set_has_tooltip(False)
+            event_item.raise_(None)
+            event_item.transparent = True
 
-        event_item.width = self._day_width - 6  # Biggest event width
-        event_date = event_item.event.start.date()
-        daysdelta = self._drag_start_date - event_date
-        if self.view == 'week':
-            event_item.x = event_item.left_border
-            if ((event_item.event.all_day or event_item.event.multidays)
-                    and self._drag_start_date != event_date):
-                event_item.x += daysdelta.days * self._day_width
-                event_item.event.start += daysdelta
-                if event_item.event.end:
-                    event_item.event.end += daysdelta
-            else:
+            event_item.width = self._day_width - 6  # Biggest event width
+            event_date = event_item.event.start.date()
+            daysdelta = self._drag_start_date - event_date
+            if self.view == 'week':
+                event_item.x = event_item.left_border
+                if ((event_item.event.all_day or event_item.event.multidays)
+                        and self._drag_start_date != event_date):
+                    event_item.x += daysdelta.days * self._day_width
+                    event_item.event.start += daysdelta
+                    if event_item.event.end:
+                        event_item.event.end += daysdelta
+                else:
+                    for item in event_item.event.event_items:
+                        if item != event_item:
+                            item.remove()
+                            self._event_items.remove(item)
+
+                    event_item.height = 2 * self._line_height
+                    day_no = (int((event.x - self._timeline.width)
+                        / self._day_width))
+                    day_off = day_no * self._day_width + 2
+                    event_item.x = self._timeline.width + day_off
+                    if (event_item.no_caption or event.y < event_item.y
+                            or event.y > (event_item.y + event_item.height)):
+                        # click was not performed inside the new day item
+                        level_height = self.minute_height * self.MIN_PER_LEVEL
+                        cur_level = int((event.y - self._timeline.y)
+                            / level_height)
+                        nb_levels_per_hour = 60 / self.MIN_PER_LEVEL
+                        # click is in the middle
+                        cur_level -= nb_levels_per_hour
+                        if cur_level < 0:
+                            cur_level = 0
+                        event_item.y = (
+                            self._timeline.y + cur_level * level_height)
+                        nb_minutes = cur_level * self.MIN_PER_LEVEL
+                        hours, minutes = map(int, divmod(nb_minutes, 60))
+                        old_start = event_item.event.start
+                        new_start = \
+                            datetime.datetime.combine(self._drag_start_date,
+                            datetime.time(hours, minutes))
+                        event_item.event.start = new_start
+                        delta = new_start - old_start
+                        if event_item.event.end:
+                            event_item.event.end += delta
+                    event_item.no_caption = False
+            elif self.view == 'month':
                 for item in event_item.event.event_items:
                     if item != event_item:
                         item.remove()
                         self._event_items.remove(item)
-
-                event_item.height = 2 * self._line_height
-                day_no = (int((event.x - self._timeline.width)
-                    / self._day_width))
-                day_off = day_no * self._day_width + 2
-                event_item.x = self._timeline.width + day_off
-                if (event_item.no_caption or event.y < event_item.y
-                        or event.y > (event_item.y + event_item.height)):
-                    # click was not performed inside the new day item
-                    level_height = self.minute_height * self.MIN_PER_LEVEL
-                    cur_level = int((event.y - self._timeline.y)
-                        / level_height)
-                    nb_levels_per_hour = 60 / self.MIN_PER_LEVEL
-                    cur_level -= nb_levels_per_hour  # click is in the middle
-                    if cur_level < 0:
-                        cur_level = 0
-                    event_item.y = self._timeline.y + cur_level * level_height
-                    nb_minutes = cur_level * self.MIN_PER_LEVEL
-                    minutes = nb_minutes % 60
-                    hours = nb_minutes / 60
-                    old_start = event_item.event.start
-                    new_start = \
-                        datetime.datetime.combine(self._drag_start_date,
-                        datetime.time(hours, minutes))
-                    event_item.event.start = new_start
-                    delta = new_start - old_start
-                    if event_item.event.end:
-                        event_item.event.end += delta
-                event_item.no_caption = False
-        elif self.view == 'month':
-            for item in event_item.event.event_items:
-                if item != event_item:
-                    item.remove()
-                    self._event_items.remove(item)
-                else:
-                    event_item.event.start += daysdelta
-                    if event_item.event.end:
-                        event_item.event.end += daysdelta
-                    weekno = int(event.y / self._day_height)
-                    day_no = int(event.x / self._day_width)
-                    event_item.y = weekno * self._day_height
-                    event_item.y += int(self._line_height) + 1  # padding-top
-                    event_item.x = day_no * self._day_width + 2  # padding-left
-                    item_height = self._line_height + 2  # 2px between items
-                    while event_item.y < event.y:
-                        event_item.y += item_height
-                    event_item.y -= item_height
-                    event_item.no_caption = False
-        event_item.update()
+                    else:
+                        event_item.event.start += daysdelta
+                        if event_item.event.end:
+                            event_item.event.end += daysdelta
+                        weekno = int(event.y / self._day_height)
+                        day_no = int(event.x / self._day_width)
+                        event_item.y = weekno * self._day_height
+                        event_item.y += (
+                            int(self._line_height) + 1)  # padding-top
+                        event_item.x = (
+                            day_no * self._day_width + 2)  # padding-left
+                        item_height = (
+                            self._line_height + 2)  # 2px between items
+                        while event_item.y < event.y:
+                            event_item.y += item_height
+                        event_item.y -= item_height
+                        event_item.no_caption = False
+            event_item.update()
         self.emit('event-pressed', event_item.event)
 
         if self._is_double_click(event):
@@ -1040,7 +1049,7 @@ class DayItem(GooCanvas.CanvasGroup):
         week_day = self.date.weekday()
         day_name = calendar.day_name[week_day]
         caption = '%s %s' % (self.date.day, day_name)
-        self.text.set_property('font-desc', self._cal.props.font_desc)
+        self.text.set_property('font', self._cal.props.font)
         self.text.set_property('text', caption)
         logical_height = self.text.get_natural_extents()[1].height
         line_height = int(math.ceil(float(logical_height) / Pango.SCALE))
@@ -1132,7 +1141,7 @@ class EventItem(GooCanvas.CanvasGroup):
         # Create canvas items.
         self.box = GooCanvas.CanvasRect(parent=self)
         self.text = GooCanvas.CanvasText(parent=self)
-        self.text.set_property('font-desc', self._cal.props.font_desc)
+        self.text.set_property('font', self._cal.props.font)
         logical_height = self.text.get_natural_extents()[1].height
         self.line_height = logical_height / Pango.SCALE
 
@@ -1315,6 +1324,8 @@ class TimelineItem(GooCanvas.CanvasGroup):
 
         if self.x is not None:
             self.update()
+        else:
+            self._compute_width()
 
     @property
     def min_line_height(self):
@@ -1333,21 +1344,20 @@ class TimelineItem(GooCanvas.CanvasGroup):
     def line_height(self):
         self.padding_top = 0
         line_height = self.min_line_height
-        if line_height < self.height / 24:
-            line_height = self.height / 24
-            pango_size = self._cal.props.font_desc.get_size()
-            padding_top = (line_height - pango_size / Pango.SCALE) / 2
+        if line_height < self.height // 24:
+            line_height = self.height // 24
+            padding_top = (line_height - self._cal.font_size / Pango.SCALE) / 2
             padding_top -= int(math.ceil(
                     float(self.ink_padding_top) / Pango.SCALE))
             self.padding_top = padding_top
         return line_height
 
     def _compute_width(self):
-        font_desc = self._cal.props.font_desc
+        font = self._cal.props.font
         ink_padding_left = 0
         ink_max_width = 0
         for n in range(24):
-            self._timeline_text[n].set_property('font-desc', font_desc)
+            self._timeline_text[n].set_property('font', font)
             natural_extents = self._timeline_text[n].get_natural_extents()
             ink_rect = natural_extents[0]
             ink_padding_left = max(ink_padding_left, ink_rect.x)
